;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Punch, from Wario Land II
; By: Schwa
;
; NOTE: You must use this sprite with the Status Condition sprite in the level, or it will
; not work properly.
;
; What this does:
; This fellow hops energetically along the ground, following Mario. When it sees him, it
; delivers a grizzly-strength straight punch, which lasts until Mario dodges or is hit.
; If the punch hits Mario, he will be sent sailing through the air.
;
; This sprite can be jumped on, but this won't kill it, it'll knock it back as it ducks
; under its helmet. Fireballs and bumping it from behind have the same effect. Only a star,
; thrown object or Cape can defeat this enemy.
;
; Extra bit:
; This sprite does not use the Extra Bit.
;
; Customizability:
; Change the "Var" variables to edit various ways the sprite behaves. Anything marked "Ram"
; should be left as it is.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Var_HIT_YSPEED = $80	; Mario's y speed when sent flying
Var_RANGE1 = $40	; how close Mario needs to be for the sprite to punch
Var_PROX_TIMER = $12	; minimum time-out in between attacks
Var_STUN_TIMER = $20	; how long the sprite will be stunned when stunned
Var_ATK_TIMER = $10	; how long the sprite will continue punching when it is out of range

Ram_STATE = $C2
Ram_PROX_TIMER = $1540
Ram_ACTION_TIMER = $163E

;;;;;;;;;;;;;;;;
; Init Routine ;
;;;;;;;;;;;;;;;;

	dcb "INIT"
	JSR SUB_HORZ_POS  ; \
	TYA		  ;  | face Mario on Init
	STA $157C,x	  ; /
	RTL

;;;;;;;;;;;;;;;;
; Main Routine ;
;;;;;;;;;;;;;;;;

	dcb "MAIN"
	PHB
	PHK
	PLB
	JSR MAIN_CODE
	PLB
	RTL

;;;;;;;;;;;;;;;;;;;;
; Main sprite code ;
;;;;;;;;;;;;;;;;;;;;

X_SPEED:
	dcb $0C,$F4

RETURN1:
	RTS

MAIN_CODE:
	JSR SUB_GFX	; call gfx routine

	LDA $14C8,x	; \
	CMP #$08	;  | if sprite is dead, do nothing
	BNE RETURN1	; /

	LDA $9D		; \ if sprites are locked, do nothing
	BNE RETURN1	; /

	JSR SUB_OFF_SCREEN_X0	; handle when off screen

	LDA $1528,x	; \ 
	CMP #$01	;  | check to see if the sprite has lost a hit point
	BNE NOT_DAMAGED	; /

	STZ $1528,x	; restore hit points to full

	LDA #$01	; \ set sprite state to 01 "stunned"
	STA Ram_STATE,x	; /

	LDA #Var_STUN_TIMER	; \ set timer for being stunned
	STA Ram_ACTION_TIMER,x	; /

	JSR SUB_HORZ_POS  ; \
	TYA		  ;  | face Mario
	STA $157C,x	  ; /

	LDA #$09	; \ sound effect
	STA $1DFC	; /
NOT_DAMAGED:
	LDA $1588,x		; \
	AND #$03		;  | check if sprite is touching the walls
	BEQ NO_TOUCH_WALLS	; /

	LDA $157C,x	; \
	EOR #$01	;  | reverse direction
	STA $157C,x	; /
NO_TOUCH_WALLS:
	LDA Ram_STATE,x	  ; \
	CMP #$01	  ;  | decide which code to call based on Sprite State
	BEQ STATE_STUNNED ;  |
	CMP #$02	  ;  |
	BEQ STATE_ATTACK  ; /

	LDA $1588,x	; \
	AND #$04	;  | check if the sprite is in the air
	BEQ IN_AIR	; /

	LDA #$F0	; \ set hopping y speed
	STA $AA,x	; /

	INC $1570,x	; increase number of total hops

	LDA $1570,x	  ; \
	AND #$07	  ;  | check if it's the eighth hop
	BNE NO_CHANGE_DIR ; /

	JSR SUB_HORZ_POS  ; \
	TYA		  ;  | face Mario
	STA $157C,x	  ; /
NO_CHANGE_DIR:
	JSR PROXIMITY	  ; \ attack Mario if he is in range
	BEQ NOT_ATTACKING ; /

	LDA #$02	; \ set sprite state to 02 "attacking"
	STA Ram_STATE,x	; /

	LDA #Var_ATK_TIMER	; \ set timer for attack duration
	STA Ram_ACTION_TIMER,x	; /

	LDA #$54	; \ sound effect
	STA $1DFC	; /
NOT_ATTACKING:
	LDY $157C,x	; \
	LDA X_SPEED,y	;  | set x speed based on direction
	STA $B6,x	; /

	JSR SUB_HORZ_POS	; \
	TYA			;  | check if the sprite is NOT facing mario
	CMP $157C,x		;  |
	BEQ IN_AIR		; /

	LDA #Var_PROX_TIMER	; \ reset Proximity Allowance timer
	STA Ram_PROX_TIMER,x	; /
IN_AIR:
	JMP DO_UPDATES

;;;;;;;;;;;;;;;;;;;;;;;
; Stunned Sprite Code ;
;;;;;;;;;;;;;;;;;;;;;;;

STUN_X_SPEED:
	dcb $DC,$24	; right, left facing (it slides backward)

STATE_STUNNED:
	LDA Ram_ACTION_TIMER,x	; \ check to see if the sprite is finished being stunned
	BNE STILL_STUNNED	; /

	STZ Ram_STATE,x	; set sprite state to 00 "normal"

	JMP DO_UPDATES

STILL_STUNNED:
	DEC Ram_ACTION_TIMER,x	; decrement timer by 1

	LDY $157C,x		; \
	LDA STUN_X_SPEED,y	;  | set x speed based on facing direction (it slides backward)
	STA $B6,x		; /

	JMP DO_UPDATES

;;;;;;;;;;;;;;;;;;;;;;;;;
; Attacking Sprite Code ;
;;;;;;;;;;;;;;;;;;;;;;;;;

ATK_X_SPEED:
	dcb $20,$E0	; right, left facing

STATE_ATTACK:
	LDA Ram_ACTION_TIMER,x	; \ check to see if the sprite is finished being stunned
	BNE STILL_ATTACKING	; /

	STZ Ram_STATE,x		; set sprite state to 00 "normal"

	JMP DO_UPDATES

STILL_ATTACKING:
	LDA #$01
	STA $1570,x		; set no. of hops so it doesn't turn around right away

	DEC Ram_ACTION_TIMER,x	; decrement timer by 1

	JSR SUB_HORZ_POS	; \
	TYA			;  | if facing Mario...
	CMP $157C,x		;  |
	BNE NO_RESET_ATK	; /

	LDA #Var_ATK_TIMER	; \ ...reset the timer so it keeps attacking
	STA Ram_ACTION_TIMER,x	; /      (it should only stop attacking if it's passed by Mario for enough frames)

NO_RESET_ATK:
	LDY $157C,x		; \
	LDA ATK_X_SPEED,y	;  | set x speed based on facing direction
	STA $B6,x		; /

	JMP DO_UPDATES

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Update speed/interaction routines ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

HIT_X_SPEED:
	dcb $6F,$90	; x speed of Mario when punched, based on sprite's facing direction (right, left)
BOUNCE_X_SPEED:
	dcb $EC,$14	; x speed to give mario when he stomps the sprite
KILLED_X_SPEED:
	dcb $F0,$10

RETURN2:
	RTS

DO_UPDATES:
	JSL $01802A	; update position based on speed values

	JSL $018032	; interact with other sprites
	JSL $01A7DC	; interact with mario
	BCC RETURN2	; do nothing if no interaction

        LDA $1490	; \ if mario star timer > 0 ...
        BNE HAS_STAR	; /    ... goto HAS_STAR
        LDA $154C,x	; \ if sprite invincibility timer > 0 ...
        BNE RETURN2	; /    ... goto NO_CONTACT
        LDA #$08	; \ sprite invincibility timer = $08
        STA $154C,x	; /
        LDA $7D		; \  if mario's y speed < 10 ...
        CMP #$10	;  |   ... sprite will hurt mario
        BMI SPRITE_WINS	; /

	LDA $15		; Save controller state
        PHA
        ORA #$C0	; Set holding X/Y and A/B
        STA $15
        JSL $01AA33	; set mario speed
	PLA
	STA $15

        JSR SUB_HORZ_POS  ; If there was contact,
	LDY $157C,x
        LDA BOUNCE_X_SPEED,y	; bounce away horizontally
        STA $7B

	INC $1528,x	; remove one hit point (this will trigger Stunned Mode)
RETURN3:
	RTS	; finish the code

SPRITE_WINS:
	LDA $1497	; \ if mario is invincible...
        ORA $187A	;  |  ... or mario on yoshi...
        BNE RETURN3	; /   ... return

	JSR SUB_HORZ_POS  ; \
	TYA		  ;  | check if the sprite is facing Mario
	CMP $157C,x	  ;  |
	BEQ HIT_MARIO	  ; /
MARIO_WINS_SIDE:
	INC $1528,x	; ...and remove a hit point, to send the sprite into Stunned Mode

	LDA $7B		; \
	EOR #$FF	;  | reverse Mario's x speed to create a bounce effect
	STA $7B		; /
	RTS

HIT_MARIO:
	LDA Ram_STATE,x		; \
	CMP #$02		;  | sprite must be Attacking for the punch to connect
	BNE MARIO_WINS_SIDE	; /

	LDA #$3F	; \ sound effect
	STA $1DFC	; /

	LDY $157C,x	  ; \
	LDA HIT_X_SPEED,y ;  | set Mario x speed
	STA $7B		  ; /
	LDA #Var_HIT_YSPEED	; \ set Mario y speed
	STA $7D			; /

	STZ Ram_STATE,x	; set sprite state to 00 "normal"

	LDA $157C,x	; \
	EOR #$01	;  | reverse direction
	STA $157C,x	; /

	RTS

;;;;;;;;;;;;;;;;;;;;
; Has Star routine ;
;;;;;;;;;;;;;;;;;;;;

HAS_STAR:
	LDA #$02                ; \ sprite status = 2 (being killed by star)
	STA $14C8,x             ; /
	LDA #$D0                ; \ set y speed
	STA $AA,x               ; /
	JSR SUB_HORZ_POS        ; get new sprite direction
	LDA KILLED_X_SPEED,y    ; \ set x speed based on sprite direction
	STA $B6,x               ; /
	INC $18D2               ; increment number consecutive enemies killed
	LDA $18D2               ; \
	CMP #$08                ; | if consecutive enemies stomped >= 8, reset to 8
	BCC NO_RESET2           ; |
	LDA #$08                ; |
	STA $18D2               ; /   
NO_RESET2:
	JSL $02ACE5             ; give mario points
	LDY $18D2               ; \ 
	CPY #$08                ; | if consecutive enemies stomped < 8 ...
	BCS NO_SOUND2           ; |
	LDA STAR_SOUNDS,y       ; |    ... play sound effect
	STA $1DF9               ; /
NO_SOUND2:
	RTS                     ; final return


;;;;;;;;;;;;;;;;;;
; points routine ;
;;;;;;;;;;;;;;;;;;

STAR_SOUNDS:
	dcb $00,$13,$14,$15,$16,$17,$18,$19

SUB_STOMP_PTS:
	PHY                     ; 
	LDA $1697               ; \
	CLC                     ;  } 
	ADC $1626,x             ; / some enemies give higher pts/1ups quicker??
	INC $1697               ; increase consecutive enemies stomped
	TAY                     ;
	INY                     ;
	CPY #$08                ; \ if consecutive enemies stomped >= 8 ...
	BCS NO_SOUND            ; /    ... don't play sound 
	LDA STAR_SOUNDS,y       ; \ play sound effect
	STA $1DF9               ; /   
NO_SOUND:
	TYA                     ; \
	CMP #$08                ;  | if consecutive enemies stomped >= 8, reset to 8
	BCC NO_RESET            ;  |
	LDA #$08                ; /
NO_RESET:
	JSL $02ACE5             ; give mario points
	PLY                     ;
	RTS                     ; return

;;;;;;;;;;;;;;;;;;;;;
; Proximity Routine ;
;;;;;;;;;;;;;;;;;;;;;

EORTBLI	dcb $FF,$00
		
PROXIMITY
	JSR SUB_HORZ_POS	; \
	TYA			;  |
	CMP $157C,x		;  | Mario is "out of range" if the sprite can't see him
	BNE PRange_Out		; /

	LDA Ram_PROX_TIMER,x	; \ Mario is out of range if the timer has time on it
	BNE DecTimer		; /

	LDA $14E0,x		;sprite x high
	XBA
	LDA $E4,x		;sprite x low
	REP #$20		;16bitA
	SEC
	SBC $94			;sub mario x
	SEP #$20		;8bitA
	PHA			;preserve for routine jump
	JSR SUB_HORZ_POS	;horizontal distance
	PLA			;restore
	EOR EORTBLI,y		;invert if needed
	CMP #Var_RANGE1		;variable
	BCS PRange_Out		;return not within range
	LDA #$01		;Z = 0
	RTS
DecTimer
	DEC Ram_PROX_TIMER,x
PRange_Out
	LDA #$00		;Z = 1
	RTS

;;;;;;;;;;;;;;;;;;;;
; Graphics Routine ;
;;;;;;;;;;;;;;;;;;;;

TILEMAP:
	dcb $20,$64,$22,$64,$20,$24
	dcb $20,$64,$22,$64,$20,$24
HORIZ_DISP:
	dcb $00,$F8,$00,$F8,$00,$F8
	dcb $00,$08,$00,$08,$00,$08
PROPERTIES:
	dcb $40,$00

SUB_GFX:
	JSR GET_DRAW_INFO	; after: Y = index to sprite tile map ($300)
				;      $00 = sprite x position relative to screen boarder 
				;      $01 = sprite y position relative to screen boarder  
	LDA $1602,x
	STA $03                 ; | $03 = index to frame start (0 or 4)                   
                   
	LDA $157C,x             ; \ $02 = sprite direction
	STA $02                 ; /
	BNE NO_ADD
	LDA $03
	CLC
	ADC #$06
	STA $03
NO_ADD:
	LDA Ram_STATE,x
	CMP #$01		; state 01 is "stunned", state 02 is "attacking"
	BNE NO_DRAW_STUNNED

	LDA $03
	CLC
	ADC #$02
	STA $03

	BRA DRAW_SPRITE

NO_DRAW_STUNNED:
	CMP #$02
	BNE DRAW_SPRITE

	LDA $03
	CLC
	ADC #$04
	STA $03

DRAW_SPRITE:
	PHX                     ; push sprite index

	LDX #$01                ; loop counter = (number of tiles per frame) - 1
LOOP_START:
	PHX                     ; push current tile number
	TXA                     ; \ X = index to horizontal displacement
	ORA $03                 ; / get index of tile (index to first tile of frame + current tile number)
FACING_LEFT:
	TAX                     ; \ 
                    
	LDA $00                 ; \ tile x position = sprite x location ($00)
	CLC                     ;  |
	ADC HORIZ_DISP,x        ;  |
	STA $0300,y             ; /
                    
	LDA $01                 ; \ tile y position = sprite y location ($01) + tile displacement
	STA $0301,y             ; /
                    
	LDA TILEMAP,x           ; \ store tile
	STA $0302,y             ; / 
        
	LDX $02                 ; \
	LDA PROPERTIES,x        ;  | get tile properties using sprite direction
	LDX $15E9               ;  |
	ORA $15F6,x             ;  | get palette info
	ORA $64                 ;  | put in level properties
	STA $0303,y             ; / store tile properties
                    
	PLX                     ; \ pull, X = current tile of the frame we're drawing
	INY                     ;  | increase index to sprite tile map ($300)...
	INY                     ;  |    ...we wrote 1 16x16 tile...
	INY                     ;  |    ...sprite OAM is 8x8...
	INY                     ;  |    ...so increment 4 times
	DEX                     ;  | go to next tile of frame and loop
	BPL LOOP_START          ; / 

	PLX                     ; pull, X = sprite index
	LDY #$02                ; \ 02, because we didn't write to 460 yet
	LDA #$01                ;  | A = number of tiles drawn - 1
	JSL $01B7B3             ; / don't draw if offscreen
	RTS                     ; return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; GET_DRAW_INFO
; This is a helper for the graphics routine.  It sets off screen flags, and sets up
; variables.  It will return with the following:
;
;		Y = index to sprite OAM ($300)
;		$00 = sprite x position relative to screen boarder
;		$01 = sprite y position relative to screen boarder  
;
; It is adapted from the subroutine at $03B760
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SPR_T1              dcb $0C,$1C
SPR_T2              dcb $01,$02

GET_DRAW_INFO       STZ $186C,x             ; reset sprite offscreen flag, vertical
                    STZ $15A0,x             ; reset sprite offscreen flag, horizontal
                    LDA $E4,x               ; \
                    CMP $1A                 ;  | set horizontal offscreen if necessary
                    LDA $14E0,x             ;  |
                    SBC $1B                 ;  |
                    BEQ ON_SCREEN_X         ;  |
                    INC $15A0,x             ; /

ON_SCREEN_X         LDA $14E0,x             ; \
                    XBA                     ;  |
                    LDA $E4,x               ;  |
                    REP #$20                ;  |
                    SEC                     ;  |
                    SBC $1A                 ;  | mark sprite invalid if far enough off screen
                    CLC                     ;  |
                    ADC.W #$0040            ;  |
                    CMP.W #$0180            ;  |
                    SEP #$20                ;  |
                    ROL A                   ;  |
                    AND #$01                ;  |
                    STA $15C4,x             ; / 
                    BNE INVALID             ; 
                    
                    LDY #$00                ; \ set up loop:
                    LDA $1662,x             ;  | 
                    AND #$20                ;  | if not smushed (1662 & 0x20), go through loop twice
                    BEQ ON_SCREEN_LOOP      ;  | else, go through loop once
                    INY                     ; / 
ON_SCREEN_LOOP      LDA $D8,x               ; \ 
                    CLC                     ;  | set vertical offscreen if necessary
                    ADC SPR_T1,y            ;  |
                    PHP                     ;  |
                    CMP $1C                 ;  | (vert screen boundry)
                    ROL $00                 ;  |
                    PLP                     ;  |
                    LDA $14D4,x             ;  | 
                    ADC #$00                ;  |
                    LSR $00                 ;  |
                    SBC $1D                 ;  |
                    BEQ ON_SCREEN_Y         ;  |
                    LDA $186C,x             ;  | (vert offscreen)
                    ORA SPR_T2,y            ;  |
                    STA $186C,x             ;  |
ON_SCREEN_Y         DEY                     ;  |
                    BPL ON_SCREEN_LOOP      ; /

                    LDY $15EA,x             ; get offset to sprite OAM
                    LDA $E4,x               ; \ 
                    SEC                     ;  | 
                    SBC $1A                 ;  | $00 = sprite x position relative to screen boarder
                    STA $00                 ; / 
                    LDA $D8,x               ; \ 
                    SEC                     ;  | 
                    SBC $1C                 ;  | $01 = sprite y position relative to screen boarder
                    STA $01                 ; / 
                    RTS                     ; return

INVALID             PLA                     ; \ return from *main gfx routine* subroutine...
                    PLA                     ;  |    ...(not just this subroutine)
                    RTS                     ; /


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SUB_OFF_SCREEN
; This subroutine deals with sprites that have moved off screen
; It is adapted from the subroutine at $01AC0D
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
                    
SPR_T12             dcb $40,$B0
SPR_T13             dcb $01,$FF
SPR_T14             dcb $30,$C0,$A0,$C0,$A0,$F0,$60,$90		;bank 1 sizes
		            dcb $30,$C0,$A0,$80,$A0,$40,$60,$B0		;bank 3 sizes
SPR_T15             dcb $01,$FF,$01,$FF,$01,$FF,$01,$FF		;bank 1 sizes
					dcb $01,$FF,$01,$FF,$01,$00,$01,$FF		;bank 3 sizes

SUB_OFF_SCREEN_X1   LDA #$02                ; \ entry point of routine determines value of $03
                    BRA STORE_03            ;  | (table entry to use on horizontal levels)
SUB_OFF_SCREEN_X2   LDA #$04                ;  | 
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X3   LDA #$06                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X4   LDA #$08                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X5   LDA #$0A                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X6   LDA #$0C                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X7   LDA #$0E                ;  |
STORE_03			STA $03					;  |            
					BRA START_SUB			;  |
SUB_OFF_SCREEN_X0   STZ $03					; /

START_SUB           JSR SUB_IS_OFF_SCREEN   ; \ if sprite is not off screen, return
                    BEQ RETURN_35           ; /
                    LDA $5B                 ; \  goto VERTICAL_LEVEL if vertical level
                    AND #$01                ; |
                    BNE VERTICAL_LEVEL      ; /     
                    LDA $D8,x               ; \
                    CLC                     ; | 
                    ADC #$50                ; | if the sprite has gone off the bottom of the level...
                    LDA $14D4,x             ; | (if adding 0x50 to the sprite y position would make the high byte >= 2)
                    ADC #$00                ; | 
                    CMP #$02                ; | 
                    BPL ERASE_SPRITE        ; /    ...erase the sprite
                    LDA $167A,x             ; \ if "process offscreen" flag is set, return
                    AND #$04                ; |
                    BNE RETURN_35           ; /
                    LDA $13                 ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0756 VC:176 00 FL:205
                    AND #$01                ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0780 VC:176 00 FL:205
                    ORA $03                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0796 VC:176 00 FL:205
                    STA $01                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0820 VC:176 00 FL:205
                    TAY                     ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0844 VC:176 00 FL:205
                    LDA $1A                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0858 VC:176 00 FL:205
                    CLC                     ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0882 VC:176 00 FL:205
                    ADC SPR_T14,y           ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0896 VC:176 00 FL:205
                    ROL $00                 ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizcHC:0928 VC:176 00 FL:205
                    CMP $E4,x               ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:0966 VC:176 00 FL:205
                    PHP                     ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:0996 VC:176 00 FL:205
                    LDA $1B                 ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdizCHC:1018 VC:176 00 FL:205
                    LSR $00                 ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdiZCHC:1042 VC:176 00 FL:205
                    ADC SPR_T15,y           ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdizcHC:1080 VC:176 00 FL:205
                    PLP                     ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F0 P:eNvMXdizcHC:1112 VC:176 00 FL:205
                    SBC $14E0,x             ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1140 VC:176 00 FL:205
                    STA $00                 ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:1172 VC:176 00 FL:205
                    LSR $01                 ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:1196 VC:176 00 FL:205
                    BCC SPR_L31             ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZCHC:1234 VC:176 00 FL:205
                    EOR #$80                ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZCHC:1250 VC:176 00 FL:205
                    STA $00                 ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1266 VC:176 00 FL:205
SPR_L31             LDA $00                 ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1290 VC:176 00 FL:205
                    BPL RETURN_35           ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1314 VC:176 00 FL:205
ERASE_SPRITE        LDA $14C8,x             ; \ if sprite status < 8, permanently erase sprite
                    CMP #$08                ; |
                    BCC KILL_SPRITE         ; /    
                    LDY $161A,x             ;A:FF08 X:0007 Y:0001 D:0000 DB:01 S:01F3 P:envMXdiZCHC:1108 VC:059 00 FL:2878
                    CPY #$FF                ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdiZCHC:1140 VC:059 00 FL:2878
                    BEQ KILL_SPRITE         ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdizcHC:1156 VC:059 00 FL:2878
                    LDA #$00                ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdizcHC:1172 VC:059 00 FL:2878
                    STA $1938,y             ;A:FF00 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdiZcHC:1188 VC:059 00 FL:2878
KILL_SPRITE         STZ $14C8,x             ; erase sprite
RETURN_35           RTS                     ; return

VERTICAL_LEVEL      LDA $167A,x             ; \ if "process offscreen" flag is set, return
                    AND #$04                ; |
                    BNE RETURN_35           ; /
                    LDA $13                 ; \
                    LSR A                   ; | 
                    BCS RETURN_35           ; /
                    LDA $E4,x               ; \ 
                    CMP #$00                ;  | if the sprite has gone off the side of the level...
                    LDA $14E0,x             ;  |
                    SBC #$00                ;  |
                    CMP #$02                ;  |
                    BCS ERASE_SPRITE        ; /  ...erase the sprite
                    LDA $13                 ;A:0000 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:1218 VC:250 00 FL:5379
                    LSR A                   ;A:0016 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1242 VC:250 00 FL:5379
                    AND #$01                ;A:000B X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1256 VC:250 00 FL:5379
                    STA $01                 ;A:0001 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1272 VC:250 00 FL:5379
                    TAY                     ;A:0001 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1296 VC:250 00 FL:5379
                    LDA $1C                 ;A:001A X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0052 VC:251 00 FL:5379
                    CLC                     ;A:00BD X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0076 VC:251 00 FL:5379
                    ADC SPR_T12,y           ;A:00BD X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0090 VC:251 00 FL:5379
                    ROL $00                 ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:enVMXdizCHC:0122 VC:251 00 FL:5379
                    CMP $D8,x               ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0160 VC:251 00 FL:5379
                    PHP                     ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0190 VC:251 00 FL:5379
                    LDA.W $001D             ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F2 P:eNVMXdizcHC:0212 VC:251 00 FL:5379
                    LSR $00                 ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:enVMXdiZcHC:0244 VC:251 00 FL:5379
                    ADC SPR_T13,y           ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:enVMXdizCHC:0282 VC:251 00 FL:5379
                    PLP                     ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:envMXdiZCHC:0314 VC:251 00 FL:5379
                    SBC $14D4,x             ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0342 VC:251 00 FL:5379
                    STA $00                 ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0374 VC:251 00 FL:5379
                    LDY $01                 ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0398 VC:251 00 FL:5379
                    BEQ SPR_L38             ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0422 VC:251 00 FL:5379
                    EOR #$80                ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0438 VC:251 00 FL:5379
                    STA $00                 ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0454 VC:251 00 FL:5379
SPR_L38             LDA $00                 ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0478 VC:251 00 FL:5379
                    BPL RETURN_35           ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0502 VC:251 00 FL:5379
                    BMI ERASE_SPRITE        ;A:8AFF X:0002 Y:0000 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0704 VC:184 00 FL:5490

SUB_IS_OFF_SCREEN   LDA $15A0,x             ; \ if sprite is on screen, accumulator = 0 
                    ORA $186C,x             ; |  
                    RTS                     ; / return


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SUB_HORZ_POS
; This routine determines which side of the sprite Mario is on.  It sets the Y register
; to the direction such that the sprite would face Mario
; It is ripped from $03B817
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SUB_HORZ_POS		LDY #$00				;A:25D0 X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizCHC:1020 VC:097 00 FL:31642
					LDA $94					;A:25D0 X:0006 Y:0000 D:0000 DB:03 S:01ED P:envMXdiZCHC:1036 VC:097 00 FL:31642
					SEC                     ;A:25F0 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizCHC:1060 VC:097 00 FL:31642
					SBC $E4,x				;A:25F0 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizCHC:1074 VC:097 00 FL:31642
					STA $0F					;A:25F4 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1104 VC:097 00 FL:31642
					LDA $95					;A:25F4 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1128 VC:097 00 FL:31642
					SBC $14E0,x				;A:2500 X:0006 Y:0000 D:0000 DB:03 S:01ED P:envMXdiZcHC:1152 VC:097 00 FL:31642
					BPL SPR_L16             ;A:25FF X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1184 VC:097 00 FL:31642
					INY                     ;A:25FF X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1200 VC:097 00 FL:31642
SPR_L16				RTS                     ;A:25FF X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:1214 VC:097 00 FL:31642

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SUB_VERT_POS
; This routine determines if Mario is above or below the sprite.  It sets the Y register
; to the direction such that the sprite would face Mario
; It is ripped from $03B829
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SUB_VERT_POS		LDY #$00               ;A:25A1 X:0007 Y:0001 D:0000 DB:03 S:01EA P:envMXdizCHC:0130 VC:085 00 FL:924
					LDA $96                ;A:25A1 X:0007 Y:0000 D:0000 DB:03 S:01EA P:envMXdiZCHC:0146 VC:085 00 FL:924
					SEC                    ;A:2546 X:0007 Y:0000 D:0000 DB:03 S:01EA P:envMXdizCHC:0170 VC:085 00 FL:924
					SBC $D8,x              ;A:2546 X:0007 Y:0000 D:0000 DB:03 S:01EA P:envMXdizCHC:0184 VC:085 00 FL:924
					STA $0F                ;A:25D6 X:0007 Y:0000 D:0000 DB:03 S:01EA P:eNvMXdizcHC:0214 VC:085 00 FL:924
					LDA $97                ;A:25D6 X:0007 Y:0000 D:0000 DB:03 S:01EA P:eNvMXdizcHC:0238 VC:085 00 FL:924
					SBC $14D4,x            ;A:2501 X:0007 Y:0000 D:0000 DB:03 S:01EA P:envMXdizcHC:0262 VC:085 00 FL:924
					BPL SPR_L11            ;A:25FF X:0007 Y:0000 D:0000 DB:03 S:01EA P:eNvMXdizcHC:0294 VC:085 00 FL:924
					INY                    ;A:25FF X:0007 Y:0000 D:0000 DB:03 S:01EA P:eNvMXdizcHC:0310 VC:085 00 FL:924
SPR_L11				RTS                    ;A:25FF X:0007 Y:0001 D:0000 DB:03 S:01EA P:envMXdizcHC:0324 VC:085 00 FL:924
